import pyo
from shakersynth.synth.rotor import RotorSynth
from typing import Dict  # noqa: F401


class Aircraft():
    def __init__(self, synth_engine: pyo.Server):
        self.data = {}  # type: Dict[str, float]
        self.is_running = False
        self.synth_engine = synth_engine
        self.rotor_synth = RotorSynth(self.synth_engine)

    def __getitem__(self, key):
        return self.data[key]

    def update(self, telemetry: dict):
        self.data = telemetry
        self.data["rotor_rpm"] = self.get_rotor_rpm()
        self.rotor_synth.update(self.data)

    def get_rotor_rpm(self):
        module = self.data["module"]
        rpm_percent = self.data["rotor_rpm_percent"]

        if(module == "mi-8"):
            # 95 gauge RPM == 192 real rotor RPM. [1, 2]
            return rpm_percent * 2.02105
        elif(module == "uh-1h"):
            # 90 gauge RPM == 324 real rotor RPM. [3]
            return rpm_percent * 3.6
        else:
            return 0.00000001

    def start(self):
        self.rotor_synth.start()
        self.is_running = True

    def stop(self):
        self.rotor_synth.stop()
        self.is_running = False

# 1. http://koavia.com/eng/product/helicopter/hvostovye_valy.shtml#2
# 2. https://www.pprune.org/rotorheads/221789-mil-8-mtv-mtv-1-info.html
# 3. https://apps.dtic.mil/dtic/tr/fulltext/u2/901787.pdf



#!/usr/bin/env python3

import logging
import platform
import pyo
import sys
from shakersynth.aircraft.aircraft import Aircraft
from shakersynth.receiver.shakersynth import ShakersynthReceiver
from logging import info

logging.basicConfig(level=logging.INFO)


def main():
    # Audio synthesis setup.
    server = pyo.Server(nchnls=2, duplex=0, buffersize=2048)
    server.setVerbosity(1)

    if platform.system() == "Windows":
        # Running for real with DCS, so ask which sound device has the bass
        # shakers attached.
        print(pyo.pa_list_devices())
        print("Enter device ID: ")
        device_id = int(input())
        server.setOutputDevice(device_id)
    else:
        # Developing on Linux, just send to the default device so we can listen
        # on headphones.
        server.setOutputDevice(pyo.pa_get_default_output())

    server.boot()
    server.start()

    # Receive Telemetry from the Shakersynth DCS export script.
    receiver = ShakersynthReceiver()

    # All telemetry processing and sound synthesis happens in the aircraft
    # model.
    aircraft = Aircraft(synth_engine=server)

    # On Windows, ctrl+c will not stop our process if we are blocked on
    # waiting for a for a UDP packet. However, ctrl+break will.
    if platform.system() == "Windows":
        print("*" * 80)
        print("Press ctrl+break to exit Shakersynth.")
        print("*" * 80)

    # A little state machine to keep track of our aircraft and telemetry.
    while True:
        telemetry = receiver.get_telemetry()

        if telemetry and not aircraft.is_running:
            # Then we have just started a mission.
            info("Starting new aircraft: %s" % telemetry["module"])
            aircraft.update(telemetry)
            aircraft.start()

        elif telemetry and aircraft.is_running:
            # Then things are ticking along nicely.
            aircraft.update(telemetry)

        elif aircraft.is_running and not telemetry:
            # Then we got an empty telemetry object while running an aircraft.
            # This signifies that we left the mission or lost contact with DCS.
            info("Shutting down aircraft: %s" % aircraft["module"])
            aircraft.stop()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(0)



import pyo
from shakersynth.config import config


class RotorSynth():
    def __init__(self, server: pyo.Server):
        self.server = server

        envelope0 = pyo.CosTable([
            (0000, 0.0),
            (2000, 1.0),
            (4000, 1.0),
            (6000, 0.0),
            (8191, 0.0),
        ])

        envelope1 = pyo.CosTable([
            (1000, 0.0),
            (4000, 1.0),
            (6000, 1.0),
            (8000, 0.0),
            (8191, 0.0),
        ])

        self.lfo = pyo.Metro(time=.25, poly=1).play()

        self.vca0 = pyo.TrigEnv(self.lfo, table=envelope0, dur=0.05, mul=0.0)
        self.vca1 = pyo.TrigEnv(self.lfo, table=envelope1, dur=0.05, mul=0.0)
        self.osc0 = pyo.Sine(freq=35, mul=self.vca0)
        self.osc1 = pyo.Sine(freq=35, mul=self.vca1)

    def update(self, telemetry):
        """Update synth parameters based on telemetry."""
        rpm = telemetry["rotor_rpm"]
        if rpm == 0:
            rpm = 0.00000001

        module = telemetry["module"]

        # Revolutions per second is more useful than RPM.
        revolutions_per_second = rpm / 60.0

        if(module == "mi-8"):
            blade_count = 5
        elif(module == "uh-1h"):
            blade_count = 2
        else:
            raise NotImplementedError

        blades_per_second = revolutions_per_second * blade_count
        blade_period = 1 / blades_per_second

        # Set the synth pulse interval to trigger for each passing blade.
        self.lfo.setTime(blade_period)

        # Get louder, as well as faster with rising rotor RPM, with a
        # fractional exponential curve so you can still feel the signal at
        # lower RPMs.
        shake_volume = ((telemetry["rotor_rpm_percent"] / 100) ** 0.6)
        shake_volume *= config.global_volume

        self.vca0.setMul(shake_volume)
        self.vca1.setMul(shake_volume)

        # Lower RPM means slower blades, so each "shake" should last longer.
        shake_duration = blade_period / 6
        self.vca0.setDur(shake_duration)
        self.vca1.setDur(shake_duration)

    def start(self):
        self.osc0.out(0)
        self.osc1.out(1)

    def stop(self):
        self.osc0.stop()
        self.osc1.stop()



import setuptools

with open("README.md", "r") as readme:
    long_description = readme.read()

setuptools.setup(
    name="Shakersynth",
    version="0.0.3",
    author="Toby McLaughlin",
    author_email="toby@jarpy.net",
    description="Bass shaker synthesizer for DCS World.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jarpy/shakersynth",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Games/Entertainment :: Simulation",
        "Topic :: Multimedia :: Sound/Audio :: Sound Synthesis",
    ],
    python_requires='>=3.7',
    install_requires=[
        'func-timeout',
        'pyo',
        'pyaml',
    ]
)



import logging
import os

try:
    log_level = getattr(logging,
                        os.getenv('SHAKERSYNTH_LOG_LEVEL', 'info').upper())
except KeyError:
    log_level_name = logging.INFO

global_volume = 0.7
